package dev.peerat.parser.java.visitor;

import java.util.Map;
import java.util.Map.Entry;
import java.util.function.BiPredicate;
import java.util.function.Predicate;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.visitor.bag.ListVisitorBag;
import dev.peerat.parser.visitor.bag.VisitorBag;

public class JavaAnnotationVisitor extends JavaVisitor<Annotation>{
	
	private Predicate<String> nameFilter;
	private Predicate<String> keyFilter;
	private BiPredicate<String, Value> valueFilter; 

	public JavaAnnotationVisitor(){
		super(Annotation.class);
	}
	
	public JavaAnnotationVisitor filterName(Predicate<String> validator){
		this.nameFilter = (this.nameFilter == null) ? validator : this.nameFilter.and(validator);
		return this;
	}
	
	public JavaAnnotationVisitor hasKey(Predicate<String> validator){
		this.keyFilter = (this.keyFilter == null) ? validator : this.keyFilter.and(validator);
		return this;
	}
	
	public JavaAnnotationVisitor filterValue(BiPredicate<String, Value> validator){
		this.valueFilter = (this.valueFilter == null) ? validator : this.valueFilter.and(validator);
		return this;
	}
	
	@Override
	public VisitorBag visitElement(Annotation element){
		VisitorBag bag = new ListVisitorBag();
		if(nameFilter != null){
			if(!nameFilter.test(element.getName().getValue())) return bag;
		}
		
		if(keyFilter != null){
			Map<Token, Value> paramters = element.getParameters();
			if(paramters == null) return bag;
			for(Token token : paramters.keySet()){
				if(!keyFilter.test(token.getValue())) return bag;
			}
		}
		
		if(valueFilter != null){
			Map<Token, Value> paramters = element.getParameters();
			if(paramters == null) return bag;
			for(Entry<Token, Value> entry : paramters.entrySet()){
				if(!valueFilter.test(entry.getKey().getValue(), entry.getValue())) return bag;
			}
		}
		
		bag.compute(element);
		return bag;
	}
	
}